/**
 * SaferoomRepository.java
 *
 * Copyright (C) 2005-2006 Tommi Laukkanen
 * http://www.substanceofcode.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package com.substanceofcode.saferoom.control;

import com.substanceofcode.crypto.Base64;
import com.substanceofcode.saferoom.data.*;
import com.substanceofcode.utils.Settings;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.microedition.midlet.MIDlet;
import javax.microedition.rms.RecordStoreException;
import com.substanceofcode.crypto.RC4;

/**
 *
 * @author Tommi Laukkanen
 */
public class SaferoomRepository {
    
    private static SaferoomRepository m_repository = null;

    /** Creates a new instance of SaferoomRepository */
    private SaferoomRepository() {
    }
    
    public static SaferoomRepository getInstance() {
        if( m_repository==null ) {
            m_repository = new SaferoomRepository();
        }
        return m_repository;        
    }
    
    /** Create database string */
    public void save(
            MIDlet midlet, 
            PasswordGroupCollection groups,
            String password) {
	String repositoryData = "";
	Enumeration en = groups.elements();
	while(en.hasMoreElements() ) {
	    PasswordGroup group = (PasswordGroup) en.nextElement();
            String groupName = group.getName();
            if( groupName.length()>0 ) {
                PasswordItemCollection passwords = group.getPasswords();
                Enumeration enpass = passwords.elements();
                while(enpass.hasMoreElements()) {
                    PasswordItem pass = (PasswordItem) enpass.nextElement();
                    repositoryData += groupName + "|";
                    repositoryData += pass.getLocation() + "|";
                    repositoryData += pass.getUsername() + "|";
                    repositoryData += pass.getPassword() + "|";
                    repositoryData += pass.getComment() + "\n";
                }
            }
	}
        
        // Encrypt and encode repository data
        byte[] repositoryBytes = repositoryData.getBytes();
        RC4 rc4 = new RC4( password );
        byte[] encryptedBytes = rc4.encrypt( repositoryBytes );
        Base64 base64 = new Base64();
        String base64Repository = base64.encode(encryptedBytes);

        // Save data to record store as RC4 encrypted and BASE64 encoded string
        Settings settings = null;
        try {
            settings = Settings.getInstance(midlet);
            settings.setStringProperty("repository", base64Repository);
            settings.save(true);
        } catch (IOException ex) {
            System.out.println("IOException in " +
                    "SaferoomRepository.save(): " + ex.toString());
        } catch (RecordStoreException ex) {
            System.out.println("RecordStoreException in " +
                    "SaferoomRepository.save(): " + ex.toString());
        }
    }
    
    /** Parse repository from database entry */
    public PasswordGroupCollection load(
            MIDlet midlet, 
            String passwordPhrase,
            boolean decryptData) throws Exception {
        PasswordGroupCollection groups = new PasswordGroupCollection();
	Hashtable groupHash = new Hashtable();
        Settings settings = null;
        try {
            settings = Settings.getInstance(midlet);
        } catch (RecordStoreException ex) {
            System.out.println("RecordStoreException in " +
                    "SaferoomRepository.load(): " + ex.toString());
            throw new Exception("Unable to initialize settings instance.");
        } catch (IOException ex) {
            System.out.println("IOException in " +
                    "SaferoomRepository.load(): " + ex.toString());
            throw new Exception("Unable to initialize settings instance.");
        }
        String data = settings.getStringProperty("repository", "");
        

        // Decode and decrypt data
        Base64 base64 = new Base64();
        byte[] decodedBytes = base64.decode(data);
        RC4 rc4 = new RC4( passwordPhrase );
        byte[] decryptedBytes = rc4.decrypt(decodedBytes);
        try {
            data = new String(decryptedBytes);
        } catch (Exception ex) {
            System.err.println("Error while constructing string with utf-8 " +
                    "encoding.");
            ex.printStackTrace();
            return null;
        }
        
        // Parse data from delimeter separated string
        // Each line is one password item
	String [] lines = split(data, "\n");
        
	for(int index=0; index<lines.length; index++) {
	    String [] values = split( lines[index], "|" );
	    if(values.length==5) {
		String groupName = values[0];
                PasswordGroup group = null;

		// Check if group already exists
		if( groupHash.containsKey(groupName)==true ) {
                    
		    group = (PasswordGroup)groupHash.get( groupName );
		    groupHash.remove( groupName );
		} else {
                    group = new PasswordGroup( groupName );
                }

		// Create password item
		PasswordItem password = new PasswordItem(
			values[1],	// Location
			values[2],	// Username
			values[3],	// Password
			values[4]);	// Comment

                PasswordItemCollection passwords = group.getPasswords();
                passwords.add(password);
                group.setPasswords(passwords);
                
		// Add group to hash table
		groupHash.put(groupName, group);
	    }
	}
	
	// Create group vector
	Enumeration en = groupHash.elements();
	while( en.hasMoreElements() ) {
	    groups.add( (PasswordGroup) en.nextElement() );
	}
        
        if( groups.size()==0) {
            return null;
        }
	
        return groups;
    }

    /** 
     * Split string into multiple strings 
     * @param original      Original string
     * @param separator     Separator string in original string
     * @return              Splitted string array
     */
    static private String[] split(String original, String separator) {
        Vector nodes = new Vector();

        // Parse nodes into vector
        int index = original.indexOf(separator);
        while(index>=0) {
            nodes.addElement( original.substring(0, index) );
            original = original.substring(index+separator.length());
            index = original.indexOf(separator);
        }
        // Get the last node
        nodes.addElement( original );

        // Create splitted string array
        String[] result = new String[ nodes.size() ];
        if( nodes.size()>0 ) {
            for(int loop=0; loop<nodes.size(); loop++)
            result[loop] = (String)nodes.elementAt(loop);
        }
        return result;
    }
    
    
}
